#include <string>
#include <iostream>
#include <cstring>
#include <cmath>
#include <fstream>
#include <iomanip>
#include <time.h>
#include <conio.h>
#include <stdlib.h>

using namespace std;

struct Var //    
{
	double x, v;
};
/*//     :
Var operator+ (Var& A, Var& B);
Var operator* (double k, Var& A);*/
struct Par //   
{
	double omega0, gamma, F, omega;
};
// ,   
double Ext_force(Par& P, double t);
// ,      :

Var RHS(Var& V, Par& P, double t);

//   :
void Trapezium_step(Var& V, Par& P, double& t, double st);
// ,        :
double Exact_x(Var& V0, Par& P, double t);
double Homogeneous_x(Var& V0, Par& P, double t);
double Partial_0_x(Par& P, double t);


Var operator+ (Var& A, Var& B)
{ //    
	Var Res;
	Res.x = A.x + B.x;
	Res.v = A.v + B.v;
	return Res;
}

Var operator* (double k, Var& A)
{ //     
	Var Res;
	Res.x = k * A.x;
	Res.v = k * A.v;
	return Res;
}

void Trapezium_step(Var &V, Par &P, double &t, double st)
{
	Var V0 = V; //     
	Var F0 = RHS(V0, P, t); //      
	Var V_prev = V;
	//  ,      ,    
	do //    
	{
		//   
		V_prev = V;
		V = V0 + 0.5 * st * (F0 + RHS(V_prev, P, t + st));
	} while (fabsl(V.x - V_prev.x) + fabsl(V.v - V_prev.v) > 0.00000001);
	//        ,    ;  ,      
	t += st; //     
}

double Ext_force(Par& P, double t)
{
	//  
	return P.F * cosl(P.omega * t);
}

Var RHS(Var& V, Par& P, double t)
{
	//     
	Var Res;
	Res.x = V.v;
	Res.v = -P.gamma * V.v - pow(P.omega0, 2) * V.x + Ext_force(P, t);
	return Res;
}

double Exact_x(Var& V0, Par& P, double t)
{
	//        
	return Homogeneous_x(V0, P, t) + Partial_0_x(P, t);
}

double Homogeneous_x(Var& V0, Par& P, double t)
{
	//        
	double Omega = sqrtl(pow(P.omega0, 2) - pow(P.gamma, 2) / 4.);
	return expl(-P.gamma * t / 2.) * (V0.x * cosl(Omega * t) + ((V0.v + P.gamma * V0.x / 2.) / Omega) * sinl(Omega * t));
}

double Partial_0_x(Par& P, double t)
{
	//        
	double Omega = sqrtl(pow(P.omega0, 2) - pow(P.gamma, 2) / 4.);
	double diff = pow(P.omega0, 2) - pow(P.omega, 2);
	double exponent = expl(-P.gamma * t / 2.);
	double factor = (pow(P.omega0, 2) + pow(P.omega, 2)) / (2. * P.omega * Omega);
	return (P.F / (pow(diff, 2) + pow(P.gamma, 2) * pow(P.omega, 2))) *(diff * (cosl(P.omega * t) - exponent * cosl(Omega * t)) + P.gamma * P.omega * (sinl(P.omega * t) - factor * exponent * sinl(Omega * t)));
}

int main()
{
	Var V;
	V.x = 0.;
	V.v = 0.;
	Var V0 = V;
	Par P;
	P.omega0 = 1.;
	P.gamma = 0.05;
	P.F = 1.;
	P.omega = 1.0001;
	double t = 0.;
	double st = 0.001;
	double x_ex = 0.; //  
	double max_diff = 0.;
	//       (   )
	ofstream x_res_file("x.txt");
	//   
	ofstream x_exact_res_file("x_exact.txt");
	//   
	double t1 = st;
	double error_out_interval = 1.;
	//             error_out_interval
	int i = 0;
	int n_steps_out = 100;
	//        x   n_steps_out 
	while (t < 200) //  
	{
		//    (0, 200)
		Trapezium_step(V, P, t, st);
		//  
		x_ex = Exact_x(V0, P, t);
		//   
		i++;
		if (i == n_steps_out)
		{
			//       
			x_res_file << V.x << "   " << t <<  " \n ";
			x_exact_res_file << x_ex << "   " << t << " \n ";
			i = 0;
		}
		//     
		if (fabsl(V.x - x_ex) > max_diff)
			max_diff = fabsl(V.x - x_ex);
		if (t1 >= error_out_interval)
		{
			//  
			cout << "\ntime: " << t
				<< fixed << " error: " << setprecision(10) << max_diff << " ";
			t1 = st;
		}
		else
			t1 += st;
	}
	x_res_file.close();
	x_exact_res_file.close();
	system("pause");
	return 0;
}
